package eu.eudat.models.data.listingmodels;

import eu.eudat.data.entities.DMP;
import eu.eudat.logic.utilities.builders.XmlBuilder;
import eu.eudat.models.DataModel;
import eu.eudat.models.data.dmp.AssociatedProfile;
import eu.eudat.models.data.dmp.Organisation;
import eu.eudat.logic.utilities.helpers.LabelBuilder;
import eu.eudat.models.data.urls.DatasetUrlListing;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;


public class DataManagementPlanListingModel implements DataModel<DMP, DataManagementPlanListingModel> {
    private String id;
    private String label;
    private String project;
    private String profile;
    private Date creationTime;
    private Date modifiedTime;
    private String organisations;
    private int version;
    private int status;
    private UUID groupId;
    private List<DatasetUrlListing> datasets;
    private List<AssociatedProfile> associatedProfiles;
    private List<UserInfoListingModel> users;
    private String description;
    private String projectAbbreviation;
    private String projectId;


    public String getId() {
        return id;
    }
    public void setId(String id) {
        this.id = id;
    }

    public String getLabel() {
        return label;
    }
    public void setLabel(String label) {
        this.label = label;
    }

    public String getProject() {
        return project;
    }
    public void setProject(String project) {
        this.project = project;
    }

    public String getProfile() {
        return profile;
    }
    public void setProfile(String profile) {
        this.profile = profile;
    }

    public Date getCreationTime() {
        return creationTime;
    }
    public void setCreationTime(Date creationTime) {
        this.creationTime = creationTime;
    }

    public Date getModifiedTime() {
        return modifiedTime;
    }
    public void setModifiedTime(Date modifiedTime) {
        this.modifiedTime = modifiedTime;
    }

    public String getOrganisations() {
        return organisations;
    }
    public void setOrganisations(String organisations) {
        this.organisations = organisations;
    }

    public int getVersion() {
        return version;
    }
    public void setVersion(int version) {
        this.version = version;
    }

    public UUID getGroupId() {
        return groupId;
    }
    public void setGroupId(UUID groupId) {
        this.groupId = groupId;
    }

    public List<DatasetUrlListing> getDatasets() {
        return datasets;
    }
    public void setDatasets(List<DatasetUrlListing> datasets) {
        this.datasets = datasets;
    }

    public int getStatus() {
        return status;
    }
    public void setStatus(int status) {
        this.status = status;
    }

    public List<AssociatedProfile> getAssociatedProfiles() {
        return associatedProfiles;
    }
    public void setAssociatedProfiles(List<AssociatedProfile> associatedProfiles) {
        this.associatedProfiles = associatedProfiles;
    }

    public List<UserInfoListingModel> getUsers() {
        return users;
    }
    public void setUsers(List<UserInfoListingModel> users) {
        this.users = users;
    }

    public String getDescription() {
        return description;
    }
    public void setDescription(String description) {
        this.description = description;
    }

    public String getProjectAbbreviation() {
        return projectAbbreviation;
    }
    public void setProjectAbbreviation(String projectAbbreviation) {
        this.projectAbbreviation = projectAbbreviation;
    }

    public String getProjectId() {
        return projectId;
    }
    public void setProjectId(String projectId) {
        this.projectId = projectId;
    }

    @Override
    public DataManagementPlanListingModel fromDataModel(DMP entity) {
        this.id = entity.getId().toString();
        this.label = entity.getLabel();
        this.groupId = entity.getGroupId();
        return this;
    }

    public DataManagementPlanListingModel fromDataModelDatasets(DMP entity) {
        this.fromDataModel(entity);
        this.status = entity.getStatus();
        this.version = entity.getVersion();
        this.project = entity.getProject().getLabel();
        if (entity.getProfile() != null) this.profile = entity.getProfile().getLabel();
        this.creationTime = entity.getCreated();
        this.modifiedTime = entity.getModified();
        this.organisations = LabelBuilder.getLabel(entity.getOrganisations().stream().map(item -> new Organisation().fromDataModel(item)).collect(Collectors.toList()));
        this.datasets = entity.getDataset().stream().map(x-> new DatasetUrlListing().fromDataModel(x)).collect(Collectors.toList());
        this.users = entity.getUsers().stream().map(x -> new UserInfoListingModel().fromDataModel(x)).collect(Collectors.toList());
        this.description = entity.getDescription();
        this.projectAbbreviation = entity.getProject().getAbbreviation();
        this.projectId = entity.getProject().getId().toString();

        if (entity.getAssociatedDmps() != null && !entity.getAssociatedDmps().isEmpty()) {
            Document viewStyleDoc = XmlBuilder.fromXml(entity.getAssociatedDmps());
            Element item = (Element) viewStyleDoc.getElementsByTagName("profiles").item(0);
            this.associatedProfiles = new LinkedList<>();
            if (item != null) {
                NodeList associatedProfilesElement = item.getChildNodes();
                for (int temp = 0; temp < associatedProfilesElement.getLength(); temp++) {
                    Node associatedProfileElement = associatedProfilesElement.item(temp);
                    if (associatedProfileElement.getNodeType() == Node.ELEMENT_NODE) {
                        this.associatedProfiles.add(new AssociatedProfile().fromXml((Element) associatedProfileElement));
                    }
                }
            }
        }

        return this;
    }

    @Override
    public DMP toDataModel() {
        return null;
    }

    @Override
    public String getHint() {
        return "dataManagementPlanListingModel";
    }
}
