import { Component, Input, OnDestroy, OnInit, SimpleChanges } from '@angular/core';
import { FormArray, FormBuilder, FormGroup } from '@angular/forms';
import { DmpProfileFieldDataType } from '../../../../core/common/enum/dmp-profile-field-type';
import { DmpProfileType } from '../../../../core/common/enum/dmp-profile-type';
import { DmpProfileDefinition } from '../../../../core/model/dmp-profile/dmp-profile';

@Component({
	selector: 'app-dynamic-dmp-field-resolver',
	templateUrl: 'dynamic-dmp-field-resolver.component.html',
	styleUrls: ['./dynamic-dmp-field-resolver.component.scss']
})
export class DynamicDmpFieldResolverComponent implements OnInit, OnDestroy {


	dmpProfileFieldDataType = DmpProfileFieldDataType;
	dmpProfileTypeEnum = DmpProfileType;

	@Input() dmpProfileDefinition: DmpProfileDefinition;
	@Input() formGroup: FormGroup;

	ngOnInit(): void {
		this.createControleFields();
	}
	ngOnChanges(changes: SimpleChanges) {
		if (changes['dmpProfileDefinition'] && !changes['dmpProfileDefinition'].isFirstChange()) {
			this.createControleFields();
		}
	}

	createControleFields(): void {
		if (this.dmpProfileDefinition != null) {
			const diasableBoolean = this.formGroup.disabled;
			this.formGroup.addControl('properties', new FormBuilder().group([]));
			(<FormGroup>this.formGroup.get('properties')).addControl('fields', new FormBuilder().array([]));
			this.dmpProfileDefinition.fields.forEach(item => {
				(<FormArray>this.formGroup.get('properties').get('fields')).push(new FormBuilder().group({
					id: [{ value: item.id, disabled: diasableBoolean }],
					value: [{ value: item.value, disabled: diasableBoolean }]
				}));
			});
		}
		if(this.dmpProfileDefinition==null){
			this.formGroup.removeControl('properties');
		}
	}

	ngOnDestroy(): void {
		this.formGroup.removeControl('properties');
	}
}
