import { Component, OnInit, ViewChild } from '@angular/core';
import { MatPaginator, MatSort } from '@angular/material';
import { Router } from '@angular/router';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from '../../core/common/base/base.component';
import { DataTableRequest } from '../../core/model/data-table/data-table-request';
import { DatasetListingModel } from '../../core/model/dataset/dataset-listing';
import { ExploreDatasetCriteriaModel } from '../../core/query/explore-dataset/explore-dataset-criteria';
import { DatasetService } from '../../core/services/dataset/dataset.service';
import { IBreadCrumbComponent } from '../misc/breadcrumb/definition/IBreadCrumbComponent';
import { BreadcrumbItem } from '../misc/breadcrumb/definition/breadcrumb-item';
import { Observable } from 'rxjs';


@Component({
	selector: 'app-explore-dataset-listing-component',
	templateUrl: 'explore-dataset-listing.component.html',
	styleUrls: ['./explore-dataset-listing.component.scss'],
})
export class ExploreDatasetListingComponent extends BaseComponent implements OnInit, IBreadCrumbComponent {

	@ViewChild(MatPaginator) _paginator: MatPaginator;
	@ViewChild(MatSort) sort: MatSort;

	totalCount: number;
	listingItems: DatasetListingModel[] = [];
	exploreDatasetCriteriaModel: ExploreDatasetCriteriaModel;
	breadCrumbs: Observable<BreadcrumbItem[]>;

	constructor(
		private datasetService: DatasetService,
		private router: Router,
	) {
		super();
	}

	ngOnInit() {
		this.refresh();

		const breadCrumbs = [];
		breadCrumbs.push({ parentComponentName: null, label: 'PUBLIC DATASETS', url: "/explore" })
		this.breadCrumbs = Observable.of(breadCrumbs);
	}

	refresh() {
		if (this._paginator.pageSize === undefined) this._paginator.pageSize = 10;
		const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
		let fields: Array<string> = new Array();
		if (this.sort && this.sort.active) { fields = this.sort.direction === 'asc' ? ['+' + this.sort.active] : ['-' + this.sort.active]; }
		const request = new DataTableRequest<ExploreDatasetCriteriaModel>(startIndex, this._paginator.pageSize, { fields: fields });
		request.criteria = this.exploreDatasetCriteriaModel || this.getDefaultCriteria();
		this.datasetService.getPublicPaged(request).pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (!result) { return []; }
			if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
			this.listingItems = result.data;
		});
	}

	rowClicked(dataset: DatasetListingModel) {
		this.router.navigate(['/datasets/publicEdit/' + dataset.id]);
	}

	onCriteriaChange(event: ExploreDatasetCriteriaModel) {
		this.exploreDatasetCriteriaModel = event;
		this._paginator.pageIndex = 0;
		this.refresh();
	}

	getDefaultCriteria(): ExploreDatasetCriteriaModel {
		const defaultCriteria = new ExploreDatasetCriteriaModel();
		// if (dmpId != null) {
		// 	defaultCriteria.dmpIds.push(dmpId);
		// }
		return defaultCriteria;
	}

	pageThisEvent(event) {
		this.refresh();
	}
}
